@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,28 +15,29 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/* Main body for the LU factorization: matrix initialization and result
+ * checking */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <math.h>
 #include <starpu.h>
-#include <starpu_profiling.h>
-#include <starpu_bound.h>
 #include "xlu.h"
 #include "xlu_kernels.h"
 
-static unsigned long size = 4096;
-static unsigned nblocks = 16;
+static unsigned long size = 0;
+static unsigned nblocks = 0;
 static unsigned check = 0;
 static unsigned pivot = 0;
 static unsigned no_stride = 0;
 static unsigned profile = 0;
-static unsigned bound = 0;
-static unsigned bounddeps = 0;
-static unsigned boundprio = 0;
+unsigned bound = 0;
+unsigned bounddeps = 0;
+unsigned boundprio = 0;
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
 TYPE *A, *A_saved;
 
@@ -60,6 +61,7 @@ static void parse_args(int argc, char **argv)
 			nblocks = strtol(argv[++i], &argptr, 10);
 		}
 
+#ifndef STARPU_SIMGRID
 		else if (strcmp(argv[i], "-check") == 0)
 		{
 			check = 1;
@@ -74,6 +76,7 @@ static void parse_args(int argc, char **argv)
 		{
 			no_stride = 1;
 		}
+#endif
 
 		else if (strcmp(argv[i], "-profile") == 0)
 		{
@@ -98,6 +101,7 @@ static void parse_args(int argc, char **argv)
 		else if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
 		{
 			fprintf(stderr,"usage: lu [-size n] [-nblocks b] [-piv] [-no-stride] [-profile] [-bound] [-bounddeps] [-bounddepsprio]\n");
+			fprintf(stderr,"Default is size %lu and nblocks %u\n", size, nblocks);
 			exit(0);
 		}
 	}
@@ -166,12 +170,13 @@ void copy_matrix_into_blocks(void)
 static void init_matrix(void)
 {
 	/* allocate matrix */
-	starpu_malloc((void **)&A, (size_t)size*size*sizeof(TYPE));
+	starpu_malloc_flags((void **)&A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 	STARPU_ASSERT(A);
 
 	starpu_srand48((long int)time(NULL));
 	/* starpu_srand48(0); */
 
+#ifndef STARPU_SIMGRID
 	/* initialize matrix content */
 	unsigned long i,j;
 	for (j = 0; j < size; j++)
@@ -183,8 +188,11 @@ static void init_matrix(void)
 			/* also randomize the imaginary component for complex number cases */
 			A[i + j*size] += (TYPE)(I*starpu_drand48());
 #endif
+			if (i == j)
+				A[i + j*size] *= 100;
 		}
 	}
+#endif
 
 }
 
@@ -300,31 +308,44 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	parse_args(argc, argv);
-
-#ifdef STARPU_QUICK_CHECK
-	size /= 4;
-	nblocks /= 4;
-#endif
-
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV)
 		return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	starpu_helper_cublas_init();
+	int power = starpu_cpu_worker_get_count() + 32 * starpu_cuda_worker_get_count();
+	int power_cbrt = cbrt(power);
+#ifndef STARPU_LONG_CHECK
+	power_cbrt /= 2;
+#endif
+	if (power_cbrt < 1)
+		power_cbrt = 1;
+
+#ifdef STARPU_QUICK_CHECK
+	if (!size)
+		size = 320*2*power_cbrt;
+	if (!nblocks)
+		nblocks = 2*power_cbrt;
+#else
+	if (!size)
+		size = 960*8*power_cbrt;
+	if (!nblocks)
+		nblocks = 8*power_cbrt;
+#endif
+
+	parse_args(argc, argv);
+
+	starpu_cublas_init();
 
 	init_matrix();
 
+#ifndef STARPU_SIMGRID
 	unsigned *ipiv = NULL;
 	if (check)
 		save_matrix();
 
 	display_matrix(A, size, size, "A");
 
-	if (bound)
-		starpu_bound_start(bounddeps, boundprio);
-
 	if (profile)
 		starpu_profiling_status_set(STARPU_PROFILING_ENABLE);
 
@@ -335,7 +356,7 @@ int main(int argc, char **argv)
 		if (no_stride)
 		{
 			/* in case the LU decomposition uses non-strided blocks, we _copy_ the matrix into smaller blocks */
-			A_blocks = malloc(nblocks*nblocks*sizeof(TYPE **));
+			A_blocks = malloc(nblocks*nblocks*sizeof(TYPE *));
 			copy_matrix_into_blocks();
 
 			ret = STARPU_LU(lu_decomposition_pivot_no_stride)(A_blocks, ipiv, size, size, nblocks);
@@ -345,16 +366,16 @@ int main(int argc, char **argv)
 		}
 		else
 		{
-			struct timeval start;
-			struct timeval end;
+			double start;
+			double end;
 
-			gettimeofday(&start, NULL);
+			start = starpu_timing_now();
 
 			ret = STARPU_LU(lu_decomposition_pivot)(A, ipiv, size, size, nblocks);
 
-			gettimeofday(&end, NULL);
+			end = starpu_timing_now();
 
-			double timing = (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
+			double timing = end - start;
 
 			unsigned n = size;
 			double flop = (2.0f*n*n*n)/3.0f;
@@ -363,6 +384,7 @@ int main(int argc, char **argv)
 		}
 	}
 	else
+#endif
 	{
 		ret = STARPU_LU(lu_decomposition)(A, size, size, nblocks);
 	}
@@ -371,14 +393,11 @@ int main(int argc, char **argv)
 	{
 		FPRINTF(stderr, "Setting profile\n");
 		starpu_profiling_status_set(STARPU_PROFILING_DISABLE);
-		starpu_bus_profiling_helper_display_summary();
+		starpu_profiling_bus_helper_display_summary();
 	}
 
 	if (bound)
 	{
-		double min;
-		FPRINTF(stderr, "Setting bound\n");
-		starpu_bound_stop();
 		if (bounddeps)
 		{
 			FILE *f = fopen("lu.pl", "w");
@@ -394,29 +413,27 @@ int main(int argc, char **argv)
 			FPRINTF(stderr,"system printed to lu.mps\n");
 			fclose(f);
 		}
-		else
-		{
-			starpu_bound_compute(&min, NULL, 0);
-			if (min != 0.)
-				FPRINTF(stderr, "theoretical min: %f ms\n", min);
-		}
 	}
 
+#ifndef STARPU_SIMGRID
 	if (check)
 	{
 		FPRINTF(stderr, "Checking result\n");
-		if (pivot) {
+		if (pivot)
+		{
 			pivot_saved_matrix(ipiv);
-			free(ipiv);
 		}
 
 		check_result();
 	}
+#endif
 
-	starpu_free(A);
+	if (pivot)
+		free(ipiv);
 
-	FPRINTF(stderr, "Shutting down\n");
-	starpu_helper_cublas_shutdown();
+	starpu_free_flags(A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+
+	starpu_cublas_shutdown();
 
 	starpu_shutdown();
 
