@@ -1,13 +1,14 @@
-/*
- * StarPU
- * Copyright (C) Université Bordeaux 1, CNRS 2010 (see AUTHORS file)
+/* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * This program is free software; you can redistribute it and/or modify
+ * Copyright (C) 2010 Université Bordeaux
+ * Copyright (C) 2010, 2017  CNRS
+ *
+ * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation; either version 2.1 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * StarPU is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
@@ -27,13 +28,16 @@
  * pthread_spinlock_*
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 #include <windows.h>
 #include <sys/types.h>
-#undef interface
+#include <sys/stat.h>
+#ifndef STARPU_CONFIGURE
+#  include <starpu_config.h>
+#endif
+#ifdef STARPU_HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <time.h>
 #include <stdio.h>
 #include <errno.h>
 
@@ -55,6 +59,11 @@ extern "C" {
 #define winPthreadAssert(expr) do { if (!(expr)) return EIO; } while (0)
 #endif
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif /* __cplusplus */
+
 /***********
  * threads *
  ***********/
@@ -168,7 +177,12 @@ typedef HANDLE pthread_mutex_t;
 typedef int pthread_mutexattr_t;
 
 static __inline int pthread_mutexattr_init(pthread_mutexattr_t *attr) {
-  *attr = PTHREAD_MUTEX_RECURSIVE;
+  *attr = PTHREAD_MUTEX_ERRORCHECK;
+  return 0;
+}
+
+static __inline int pthread_mutexattr_destroy(pthread_mutexattr_t *attr) {
+  *attr = -1;
   return 0;
 }
 
@@ -180,14 +194,14 @@ static __inline int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int typ
 }
 
 static __inline int pthread_mutex_init (pthread_mutex_t *mutex, pthread_mutexattr_t *attr) {
-  if (attr && *attr!=PTHREAD_MUTEX_RECURSIVE)
+  if (attr && *attr!=PTHREAD_MUTEX_ERRORCHECK)
     return EINVAL;
-  winPthreadAssertWindows(*mutex = CreateMutex(NULL, FALSE, NULL));
+  winPthreadAssertWindows(*mutex = CreateSemaphore(NULL, 1, 1, NULL));
   return 0;
 }
 
 static __inline int pthread_mutex_unlock (pthread_mutex_t *mutex) {
-  winPthreadAssertWindows(ReleaseMutex(*mutex));
+  winPthreadAssertWindows(ReleaseSemaphore(*mutex, 1, NULL));
   return 0;
 }
 
@@ -195,7 +209,7 @@ static __inline int pthread_mutex_lock (pthread_mutex_t *mutex);
 static __inline int __pthread_mutex_alloc_concurrently (pthread_mutex_t *mutex) {
   HANDLE mutex_init_mutex;
   /* Get access to one global named mutex to serialize mutex initialization */
-  winPthreadAssertWindows((mutex_init_mutex = CreateMutex(NULL, FALSE, "StarPU mutex init")));
+  winPthreadAssertWindows((mutex_init_mutex = CreateSemaphore(NULL, 1, 1, "StarPU mutex init")));
   winPthreadAssertPthread(pthread_mutex_lock(&mutex_init_mutex));
   /* Now we are the one that can initialize it */
   if (!*mutex)
@@ -270,7 +284,7 @@ typedef struct {
 } pthread_cond_t;
 #define PTHREAD_COND_INITIALIZER { NULL, 0}
 
-#ifndef _TIMESPEC_DEFINED
+#if !defined(STARPU_HAVE_STRUCT_TIMESPEC) || defined(_MSC_VER)
 #ifndef STARPU_TIMESPEC_DEFINED
 #define STARPU_TIMESPEC_DEFINED 1
 struct timespec {
@@ -278,7 +292,7 @@ struct timespec {
   long    tv_nsec; /* Nanoseconds */
 };
 #endif /* STARPU_TIMESPEC_DEFINED */
-#endif
+#endif /* STARPU_HAVE_STRUCT_TIMESPEC */
 
 typedef unsigned pthread_condattr_t;
 
@@ -385,6 +399,7 @@ static __inline int pthread_once (pthread_once_t *once, void (*oncefun)(void)) {
 }
 
 static __inline int pthread_key_create (pthread_key_t *key, void (*freefun)(void *)) {
+  (void)freefun;
   pthread_key_t res;
   winPthreadAssertWindows((res = TlsAlloc()) != 0xFFFFFFFF);
   *key = res;
