@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2014  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2013, 2014, 2015  CNRS
  * Copyright (C) 2011  Télécom-SudParis
+ * Copyright (C) 2014  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -39,7 +40,7 @@
 #include <core/errorcheck.h>
 #include <common/barrier.h>
 #include <common/utils.h>
-#include <common/thread.h>
+#include <common/list.h>
 
 #ifdef STARPU_USE_CUDA
 #include <cuda.h>
@@ -53,15 +54,21 @@ typedef void (*_starpu_cl_func_t)(void **, void *);
 #define _STARPU_CPU_MAY_PERFORM(j)	((j)->task->cl->where & STARPU_CPU)
 #define _STARPU_CUDA_MAY_PERFORM(j)      ((j)->task->cl->where & STARPU_CUDA)
 #define _STARPU_OPENCL_MAY_PERFORM(j)	((j)->task->cl->where & STARPU_OPENCL)
+#define _STARPU_MIC_MAY_PERFORM(j)	((j)->task->cl->where & STARPU_MIC)
+#define _STARPU_SCC_MAY_PERFORM(j)	((j)->task->cl->where & STARPU_SCC)
 
-struct _starpu_data_descr {
+struct _starpu_data_descr
+{
 	starpu_data_handle_t handle;
 	enum starpu_data_access_mode mode;
 	int node;
 };
 
+#ifdef STARPU_DEBUG
+MULTILIST_CREATE_TYPE(_starpu_job, all_submitted)
+#endif
 /* A job is the internal representation of a task. */
-LIST_TYPE(_starpu_job,
+struct _starpu_job {
 
 	/* Each job is attributed a unique id. */
 	unsigned long job_id;
@@ -78,7 +85,9 @@ LIST_TYPE(_starpu_job,
 	 * the task so that we always grab the rw-lock associated to the
 	 * handles in the same order. */
 	struct _starpu_data_descr ordered_buffers[STARPU_NMAXBUFS];
+	struct _starpu_task_wrapper_dlist dep_slots[STARPU_NMAXBUFS];
 	struct _starpu_data_descr *dyn_ordered_buffers;
+	struct _starpu_task_wrapper_dlist *dyn_dep_slots;
 
 	/* If a tag is associated to the job, this points to the internal data
 	 * structure that describes the tag status. */
@@ -92,6 +101,7 @@ LIST_TYPE(_starpu_job,
 	 * the handle for this dependency, so as to remove the task from the
 	 * last_writer/readers */
 	starpu_data_handle_t implicit_dep_handle;
+	struct _starpu_task_wrapper_dlist implicit_dep_slot;
 
 	/* Indicates whether the task associated to that job has already been
 	 * submitted to StarPU (1) or not (0) (using starpu_task_submit).
@@ -108,6 +118,36 @@ LIST_TYPE(_starpu_job,
 	 */
 	unsigned terminated:2;
 
+#ifdef STARPU_OPENMP
+	/* Job is a continuation or a regular task. */
+	unsigned continuation;
+
+	/* If 0, the prepared continuation is not resubmitted automatically
+	 * when going to sleep, if 1, the prepared continuation is immediately
+	 * resubmitted when going to sleep. */
+	unsigned continuation_resubmit;
+
+	/* Callback function called when:
+	 * - The continuation starpu task is ready to be submitted again if
+	 *   continuation_resubmit = 0;
+	 * - The continuation starpu task has just been re-submitted if
+	 *   continuation_resubmit = 1. */
+	void (*continuation_callback_on_sleep)(void *arg);
+	void *continuation_callback_on_sleep_arg;
+
+	void (*omp_cleanup_callback)(void *arg);
+	void *omp_cleanup_callback_arg;
+
+	/* Job has been stopped at least once. */
+	unsigned discontinuous;
+
+	/* Cumulated execution time for discontinuous jobs */
+	struct timespec cumulated_ts;
+
+	/* Cumulated energy consumption for discontinuous jobs */
+	double cumulated_energy_consumed;
+#endif
+
 	/* The value of the footprint that identifies the job may be stored in
 	 * this structure. */
 	uint32_t footprint;
@@ -117,7 +157,7 @@ LIST_TYPE(_starpu_job,
 	 * with dot) */
 	unsigned exclude_from_dag:1;
 
-	/* Is that task internal to StarPU ? */
+	/* Is that task internal to StarPU? */
 	unsigned internal:1;
 
 	/* During the reduction of a handle, StarPU may have to submit tasks to
@@ -140,40 +180,76 @@ LIST_TYPE(_starpu_job,
 	 * parallel tasks only). */
 	int active_task_alias_count;
 
-	/* A symbol name may be associated to the job directly for debug
-	 * purposes (for instance if the codelet is NULL). */
-        const char *model_name;
+	/* Used to record codelet start time instead of using a
+	 * local variable */
+	struct timespec cl_start;
 
 	struct bound_task *bound_task;
 
 	/* Parallel workers may have to synchronize before/after the execution of a parallel task. */
 	starpu_pthread_barrier_t before_work_barrier;
 	starpu_pthread_barrier_t after_work_barrier;
-)
+	unsigned after_work_busy_barrier;
+
+	struct _starpu_graph_node *graph_node;
+
+#ifdef STARPU_DEBUG
+	/* Linked-list of all jobs, for debugging */
+	struct _starpu_job_multilist_all_submitted all_submitted;
+#endif
+};
+
+#ifdef STARPU_DEBUG
+MULTILIST_CREATE_INLINES(struct _starpu_job, _starpu_job, all_submitted)
+#endif
+
+void _starpu_job_init(void);
+void _starpu_job_fini(void);
 
 /* Create an internal struct _starpu_job *structure to encapsulate the task. */
-struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_task *task);
+struct _starpu_job* _starpu_job_create(struct starpu_task *task) STARPU_ATTRIBUTE_MALLOC;
 
 /* Destroy the data structure associated to the job structure */
 void _starpu_job_destroy(struct _starpu_job *j);
 
+/* Test for the termination of the job */
+int _starpu_job_finished(struct _starpu_job *j);
+
 /* Wait for the termination of the job */
 void _starpu_wait_job(struct _starpu_job *j);
 
+#ifdef STARPU_OPENMP
+/* Test for the termination of the job */
+int _starpu_test_job_termination(struct _starpu_job *j);
+
+/* Prepare the job for accepting new dependencies before becoming a continuation. */
+
+void _starpu_job_prepare_for_continuation_ext(struct _starpu_job *j, unsigned continuation_resubmit,
+		void (*continuation_callback_on_sleep)(void *arg), void *continuation_callback_on_sleep_arg);
+void _starpu_job_prepare_for_continuation(struct _starpu_job *j);
+void _starpu_job_set_omp_cleanup_callback(struct _starpu_job *j,
+		void (*omp_cleanup_callback)(void *arg), void *omp_cleanup_callback_arg);
+#endif
+
 /* Specify that the task should not appear in the DAG generated by debug tools. */
 void _starpu_exclude_task_from_dag(struct starpu_task *task);
 
 /* try to submit job j, enqueue it if it's not schedulable yet. The job's sync mutex is supposed to be held already */
 unsigned _starpu_enforce_deps_and_schedule(struct _starpu_job *j);
 unsigned _starpu_enforce_deps_starting_from_task(struct _starpu_job *j);
+#ifdef STARPU_OPENMP
+/* When waking up a continuation, we only enforce new task dependencies */
+unsigned _starpu_reenforce_task_deps_and_schedule(struct _starpu_job *j);
+#endif
 
-
+/* Called at the submission of the job */
+void _starpu_handle_job_submission(struct _starpu_job *j);
 /* This function must be called after the execution of a job, this triggers all
  * job's dependencies and perform the callback function if any. */
 void _starpu_handle_job_termination(struct _starpu_job *j);
 
 /* Get the sum of the size of the data accessed by the job. */
-size_t _starpu_job_get_data_size(struct starpu_perfmodel *model, enum starpu_perfmodel_archtype arch, unsigned nimpl, struct _starpu_job *j);
+size_t _starpu_job_get_data_size(struct starpu_perfmodel *model, struct starpu_perfmodel_arch* arch, unsigned nimpl, struct _starpu_job *j);
 
 /* Get a task from the local pool of tasks that were explicitly attributed to
  * that worker. */
@@ -196,4 +272,6 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 #define _STARPU_JOB_SET_ORDERED_BUFFER(job, buffer, i) do { if (job->dyn_ordered_buffers) job->dyn_ordered_buffers[i] = buffer; else job->ordered_buffers[i] = buffer;} while(0)
 #define _STARPU_JOB_GET_ORDERED_BUFFERS(job) (job->dyn_ordered_buffers) ? job->dyn_ordered_buffers : job->ordered_buffers
 
+#define _STARPU_JOB_GET_DEP_SLOTS(job) (((job)->dyn_dep_slots) ? (job)->dyn_dep_slots : (job)->dep_slots)
+
 #endif // __JOBS_H__
